from pyRealtor.geo import GeoLocationService
from pyRealtor.realtor import RealtorService
from pyRealtor.report import ReportingService
from pyRealtor.facade import HousesFacade

__version__ = "0.1.0"

__all__ = [
    "GeoLocationService",
    "RealtorService",
    "ReportingService",
    "HousesFacade"
]


import requests

class GeoLocationService:

    def __init__(self):
        self.nominatim_api_endpoint = "https://nominatim.openstreetmap.org/search"
        self.latitude_max = None
        self.latitude_min = None
        self.longitude_max = None
        self.longitude_min = None

    def set_geo_location_boundry(self, geo_coordinate_1: tuple, geo_coordinate_2: tuple):
        if not ((geo_coordinate_1[0] >= -90 and geo_coordinate_1[0] <= 90) and (geo_coordinate_2[0] >= -90 and geo_coordinate_2[0] <= 90)):
            raise ValueError("Latitude of geo co-ordinate must be between -90 and 90")
        
        if not ((geo_coordinate_1[1] >= -180 and geo_coordinate_1[1] <= 180) and (geo_coordinate_2[1] >= -180 and geo_coordinate_2[1] <= 180)):
            raise ValueError("Longitude of geo co-ordinate must be between -90 and 90")

        self.latitude_min = min(geo_coordinate_1[0], geo_coordinate_2[0])
        self.latitude_max = max(geo_coordinate_1[0], geo_coordinate_2[0])
        self.longitude_min = min(geo_coordinate_1[1], geo_coordinate_2[1])
        self.longitude_max = max(geo_coordinate_1[1], geo_coordinate_2[1])

    def set_display_physical_location(self, physical_location: str):
        self.physical_location = physical_location
           
    def search_geo_location(self, city: str, province:str = None, country:str = 'Canada'):
        geo_res_json = None
        api_params = {
            'city': city,
            'country': country,
            'format': 'json'
        }

        if province:
            api_params['state'] = province

        try:
            geo_res = requests.get(
                self.nominatim_api_endpoint,
                params = api_params,
                headers={
                    'User-Agent': 'pyRealtor'
                }
            )
            #print(geo_res)
            #print(geo_res.text)
            geo_res_json_lst = geo_res.json()
            geo_res_json = None
            max_importance = 0

            for geo_json in geo_res_json_lst:
                if "canada" in geo_json["display_name"].lower() and geo_json["importance"] >= max_importance:
                    geo_res_json = geo_json
                    max_importance = geo_json["importance"]

        except Exception as e:
            print(e)
            raise
        
        return geo_res_json


import json
import importlib.resources
import pkg_resources

import pandas as pd

class ReportingService:  

    def __init__(self, column_mapping_cfg_fpath:str = 'realtor_json_normalize_mapping.json', column_lst: list = [
        'mls', 'listing_description', 'total_bathrooms', 'total_bedrooms', 'size', 'total_stories', 
        'house_type', 'house_ammenities', 
        'listing_price', 'listing_address', 'listing_address_lat', 'listing_address_long', 'listing_ownership_type', 'lising_nearby_ammenities', 'open_house', 'listing_website'
    ]):
        self.house_json_lst = []
        self.column_lst = column_lst
        self.column_mapping_dict = self.load_mapping_cfg(
            config_path=column_mapping_cfg_fpath
        )

    def load_mapping_cfg(self, config_path:str):
        config_dict = {}
        
        with pkg_resources.resource_stream('pyRealtor', config_path) as f:
            config_dict = json.load(f)
        
        return config_dict


    def to_dataframe(self):
        dataframe_values = []
        #print(self.house_json_lst)
        for mls_listing_json in self.house_json_lst:
            dataframe_row_values = []
            for col_to_report in self.column_lst:
                json_copy = mls_listing_json.copy()
                col_hierarchy_lst = self.column_mapping_dict[col_to_report].split(".")
                for col_hierarchy in col_hierarchy_lst:
                    if col_hierarchy in json_copy:
                        json_copy = json_copy[col_hierarchy]
                    else:
                        json_copy = ''
                dataframe_row_values.append(json_copy)
            dataframe_values.append(dataframe_row_values)
        #print(dataframe_values)
        dataframe = pd.DataFrame(dataframe_values, columns=self.column_lst)
        
        return dataframe

    def save_excel(self, dataframe_to_report, file_path):
        dataframe_to_report.to_excel(file_path, index=False, freeze_panes = (1,0))


import requests

from pyRealtor.geo import GeoLocationService
from pyRealtor.report import ReportingService

class RealtorService:

    def __init__(self, report_obj: ReportingService):
        self.search_api_endpoint = "https://api2.realtor.ca/Listing.svc/PropertySearch_Post"
        self.search_api_params = {
            'Version': '7.0',
            'ApplicationId': '1',
            'CultureId': '1',
            'Currency': 'CAD',
            'RecordsPerPage': 200
        }
        self.search_api_headers = {
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-US,en;q=0.9",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "DNT": "1",
            "Host": "api2.realtor.ca",
            "Origin": "https://www.realtor.ca",
            "Pragma": "no-cache",
            "Referer": "https://www.realtor.ca/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
            'User-Agent': "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Mobile Safari/537.36"
        }
        self.report_obj = report_obj

    def set_geo_coordinate_boundry(self, geo_location_obj: GeoLocationService):
        try:
            self.search_api_params["LatitudeMin"] = geo_location_obj.latitude_min
            self.search_api_params["LongitudeMin"] = geo_location_obj.longitude_min
            self.search_api_params["LatitudeMax"] = geo_location_obj.latitude_max
            self.search_api_params["LongitudeMax"] = geo_location_obj.longitude_max
            self.search_api_params["ZoomLevel"] = 10
        except Exception as e:
            raise

    def set_sort_method(self, by: str, ascending_order: bool=True):
        if by.lower() not in ["listing_price", "listing_date_posted"]:
            raise ValueError("Expected value for sort arguments are 'listing_price' or 'listing_date_posted'")

        realtor_sorting_api_str = None

        try:
            if by.lower() == 'listing_price':
                realtor_sorting_api_str = "1"
            elif by.lower() == 'listing_date_posted':
                realtor_sorting_api_str = "6"
            
            if ascending_order:
                realtor_sorting_api_str += "-A"
            else:
                realtor_sorting_api_str += "-D"
            
            self.search_api_params["Sort"] = realtor_sorting_api_str
        except Exception as e:
            raise

    def set_transaction_type(self, transaction_type: str = 'for_sale'):
        if transaction_type.lower() not in ["for_sale", "for_rent", "for_sale_or_rent"]:
            raise ValueError(f"Possible values for Transaction Type argument are 'for_sale' or 'for_rent', however received {transaction_type}")
        
        transaction_type_to_id_dict = {
            'for_sale': '2',
            'for_rent': '3'
        }

        self.search_api_params["TransactionTypeId"] = transaction_type_to_id_dict[transaction_type.lower()]

    def set_built_year_range(self, built_year_range: tuple):
        built_year_min, built_year_max = built_year_range
        if built_year_min:
            self.search_api_params["BuildingAgeMin"] = built_year_min
        if built_year_max:
            self.search_api_params["BuildingAgeMax"] = built_year_max

    def set_open_house_only(self, open_house_date):
        self.search_api_params["OpenHouse"] = 1
        self.search_api_params["OpenHouseStartDate"] = open_house_date
        self.search_api_params["OpenHouseEndDate"] = open_house_date

    def search_houses(self):
        search_result = None
        json_search_payload = self.search_api_params.copy()
        current_page_number = 1


        try:
            s = requests.Session()

            
            
            get_api_response = s.get(
                'https://realtor.ca/',
                headers = {
                    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
                    'Accept-Encoding': 'gzip, deflate, br',
                    'Accept-Language': 'en-US,en;q=0.9',
                    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36'
                }
            )

            #print(get_api_response)

            s.headers.update(self.search_api_headers)

            #print(s.cookies.get_dict())
            

            """
            reeser84_token_res = s.post(
                "https://www.realtor.ca/dnight-Exit-shall-Braith-Then-why-vponst-is-proc",
                params={'d': 'www.realtor.ca'},
                json = {"solution":{"interrogation":None,"version":"beta"},"old_token":None,"error":None,"performance":{"interrogation":1897}}
            )
            if reeser84_token_res.status_code == 200:
                token = reeser84_token_res.json()["token"]
                s.headers.update({'reese84': token})
            print(reeser84_token_res.status_code)
            print(reeser84_token_res.text)
            sys.exit(0)
            """

            json_search_payload['CurrentPage'] = str(current_page_number)
            realtor_api_response = s.post(
                self.search_api_endpoint,
                headers=self.search_api_headers,
                data = json_search_payload
            )

            
            if realtor_api_response.status_code == 200:
                search_result = realtor_api_response.json()
                self.report_obj.house_json_lst.extend(search_result["Results"])

                

                total_available_pages = search_result["Paging"]["TotalPages"]

                #print(total_available_pages)
                while current_page_number < total_available_pages:
                    current_page_number += 1
                    json_search_payload['CurrentPage'] = str(current_page_number)
                    
                    #time.sleep(1000)

                    realtor_api_response = s.post(
                        self.search_api_endpoint,
                        headers=self.search_api_headers,
                        data = json_search_payload
                    )
                    #print(s.cookies.get_dict())

                    if realtor_api_response.status_code == 200:
                        search_result = realtor_api_response.json()
                        self.report_obj.house_json_lst.extend(search_result["Results"])
                    else:
                        print(realtor_api_response.status_code)
                        raise
        except Exception as e:
            raise

        return self.report_obj


from setuptools import setup

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name = 'pyRealtor',
    version = '0.1.2',
    description = 'Python package for fetching and analyzing REALTOR.CA MLS Listings',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    author = 'Rachit Trivedi',
    author_email = 'rachitt96@gmail.com',
    license_files = ('LICENSE.md',),
    url = 'https://github.com/rachitt96/pyRealtor',
    packages = ['pyRealtor'],
    python_requires = '>=3.6',
    install_requires = [
        'requests>=2.26.0',
        'pandas>=1.3.5',
        'numpy>=1.21.4',
        'openpyxl>=3.1.2'
    ],
    extras_required = {
        "dev": ["twine"]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    include_package_data = True,
    package_data = {
        'pyRealtor': ['config/column_mapping_cfg.json']
    }
)


import os

from pyRealtor.geo import GeoLocationService
from pyRealtor.realtor import RealtorService
from pyRealtor.report import ReportingService

class HousesFacade:

    def search_save_houses(
        self,
        search_area: str,
        report_file_name: str,
        listing_type: str = 'for_sale', 
        column_mapping_cfg_fpath:str = 'config/column_mapping_cfg.json', 
        column_lst: list = [
        'mls', 'listing_description', 'total_bathrooms', 'total_bedrooms', 'size', 'total_stories', 
        'house_type', 'house_ammenities', 
        'listing_price', 'listing_address', 'listing_address_lat', 'listing_address_long', 'listing_ownership_type', 'lising_nearby_ammenities', 'open_house', 'listing_website'],
        **kwargs
    ):
        current_directory = os.getcwd()
        file_path_to_save = os.path.join(current_directory, report_file_name)

        geo_service_obj = GeoLocationService()
        realtor_service_obj = RealtorService(
            ReportingService(
                column_mapping_cfg_fpath,
                column_lst
            )
        )

        geo_result_json = geo_service_obj.search_geo_location(city=search_area)

        """
        display_address = geo_result_json['SubArea'][0]['Location']
        geo_coord_1 = (
            geo_result_json['SubArea'][0]['Viewport']['NorthEast']['Latitude'],
            geo_result_json['SubArea'][0]['Viewport']['NorthEast']['Latitude']
        )
        geo_coord_2 = (
            geo_result_json['SubArea'][0]['Viewport']['SouthWest']['Latitude'],
            geo_result_json['SubArea'][0]['Viewport']['SouthWest']['Latitude']
        )
        """

        display_address = geo_result_json["display_name"]
        geo_coord_1 = (
            float(geo_result_json['boundingbox'][0]),
            float(geo_result_json['boundingbox'][2])
        )
        geo_coord_2 = (
            float(geo_result_json['boundingbox'][1]),
            float(geo_result_json['boundingbox'][3])
        )

        geo_service_obj.set_display_physical_location(display_address)
        geo_service_obj.set_geo_location_boundry(geo_coord_1, geo_coord_2)

        realtor_service_obj.set_geo_coordinate_boundry(geo_service_obj)
        realtor_service_obj.set_transaction_type(listing_type)

        if 'open_house_date' in kwargs:
            open_house_date = kwargs['open_house_date']
            realtor_service_obj.set_open_house_only(open_house_date)

        #print(realtor_service_obj.search_api_params)
        houses_df = realtor_service_obj.search_houses().to_dataframe()
        print(houses_df)

        if houses_df.shape[0] > 0:
            realtor_service_obj.report_obj.save_excel(
                houses_df,
                file_path_to_save
            )